/* Copyright 2010 ESRI
* 
* All rights reserved under the copyright laws of the United States
* and applicable international laws, treaties, and conventions.
* 
* You may freely redistribute and use this sample code, with or
* without modification, provided you include the original copyright
* notice and use restrictions.
* 
* See the use restrictions at http://help.arcgis.com/en/sdk/10.0/usageRestrictions.htm
* 
*/
package com.esri.arcgis.sample.bookmarks;

import com.esri.adf.web.data.geometry.WebExtent;
import java.util.Iterator;

/**
 * Interface to define collection of bookmarks
 */
public interface Bookmarks {
  /**
   * Load bookmarks
   */
  public boolean load();

  /**
   * Return true if bookmarks are editable
   */
  public boolean isEditable();

  /**
   * Save bookmarks
   */
  public boolean save();

  /**
   * Get list of bookmarks ids
   */
  public Iterator getBookmarkIds();

  /**
   * Get bookmark with argument id
   */
  public Bookmark get(String id);

  /**
   * If editable, add bookmark with argument label, description & extent. Returns
   * id of new bookmark
   */
  public String add(String label, String description, WebExtent extent);

  /**
   * Update bookmark with argument id
   */
  public void update(String id, String label, String description, WebExtent extent);

  /**
   * Remove bookmark with argument id
   */
  public Object remove(String id);
}